<?php
/*
Plugin Name: NC State Newswire Plugin
Description: Adds news posts to your own WordPress site
Version: 1.1.7
*/

namespace NCState;

require __DIR__ . '/vendor/autoload.php';

use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Request;
use NCState\Newswire\Dashboard;
use NCState\Newswire\Redirect_Manager;
use NCState\Newswire\Jobs\Import_Collection;
use NCState\Newswire\Jobs\Refresh_Collection;

defined( 'ABSPATH' ) OR exit;

register_activation_hook( __FILE__, array('NCState\Newswire', 'on_activation') );
register_deactivation_hook( __FILE__, array('NCState\Newswire', 'on_deactivation') );

class Newswire {

	/**
	 * @var NCState\Newswire A single instance of this class
	 */
	private static $instance = null;

	/**
	 * @var NCState\Newswire\Dashboard
	 */
	private $dashboard;

	/**
	 * @var array
	 */
	private $settings = array();

	/**
	 * @var array
	 */
	private $data = array();

	/**
	 * @var array
	 */
	private $default_settings = array(
		'collection_ids' => '',
		'autopublish' => 'off',
		'include_attribution' => 'off',
		'redirect_all' => 'off'
	);

	/**
	 * @var array
	 */
	private $default_data = array(
		'last_update' => false
	);

	/**
	 * @var NCState\Newswire\Jobs\Import_Collection
	 */
	protected $import_collection;

	/**
	 * @var NCState\Newswire\Jobs\Refresh_Collection
	 */
	protected $refresh_collection;

	/**
	 * Load settings and data and hook for init.
	 */
	private function __construct() {
		$this->settings = wp_parse_args( get_option('ncstate_newswire_settings'), $this->default_settings );
		$this->data = wp_parse_args( get_option('ncstate_newswire_data'), $this->default_data );

		add_action( 'plugins_loaded', array( $this, 'init' ) );
	}

	/**
	 * Creates or returns an instance of this class.
	 *
	 * @return  NCState\Newswire A single instance of this class.
	 */
	public static function get_instance() {
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}

	/**
	 * Bootstrap the plugin.
	 *
	 * @return void
	 */
	public function init() {
		require_once plugin_dir_path( __FILE__ ) . 'dashboard-settings.php';
		require_once plugin_dir_path( __FILE__ ) . 'canonical-url.php';

		// Migrate up from 1.0.2 if needed.
		if ( $last_update = get_option( 'newswire_last_update', false ) ) {
			$ids = get_option( 'newswire_collection_id', '' );
			$autopublish = get_option( 'newswire_autopublish', false ) === 'checked' ? 'on' : 'off';
			$attribution = get_option( 'newswire_article_attribution', false ) === 'checked' ? 'on' : 'off';

			$this->settings = array(
				'collection_ids' => $ids,
				'autopublish' => $autopublish,
				'include_attribution' => $attribution,
			);

			$this->set_last_update( $last_update );
			update_option( 'ncstate_newswire_settings', $this->settings );

			delete_option( 'newswire_last_update' );
			delete_option( 'newswire_collection_id' );
			delete_option( 'newswire_imported_post_ids' );
			delete_option( 'newswire_autopublish' );
			delete_option( 'newswire_article_attribution' );
		}

		if ( ! wp_next_scheduled ( 'newswire_hourly_update' ) ) {
			wp_schedule_event( time(), 'hourly', 'newswire_hourly_update' );
		}

		add_action( 'newswire_hourly_update', array( $this, 'fetch_content' ) );
		add_filter( 'the_content', '\NCState\Newswire\Post::maybe_add_attribution' );
		add_filter( 'rest_prepare_post', array( $this, 'override_content_context' ), 10, 3 );

		register_meta( 'post', 'source', array(
			'type' => 'string',
			'description' => 'Did the post come from the NC State Newswire?',
			'single' => true,
			'show_in_rest' => true
		) );

		new Redirect_Manager;
		$this->dashboard = new Dashboard;
		$this->import_collection = new Import_Collection;
		$this->refresh_collection = new Refresh_Collection;
	}

	/**
	 * Activation hook
	 *
	 * @return void
	 */
	public static function on_activation() {
		if ( ! current_user_can( 'activate_plugins' ) )
			return;

		Newswire::newswire_user_creation();
	}

	/**
	 * Deactivation hook
	 *
	 * @return void
	 */
	public static function on_deactivation() {
		wp_clear_scheduled_hook('newswire_hourly_update');
	}

	/**
	 * Adds raw content to REST response regardless of context.
	 *
	 * @return \WP_REST_Response
	 */
	public static function override_content_context( $response, $post, $request ) {
		$response->data['content']['raw'] = $post->post_content;
		return $response;
	}

	/**
	 * Queue an import for each collection.
	 *
	 * @param  bool $refresh Refresh all Newswire content when true
	 * @return void
	 */
	public function fetch_content( $refresh = false ) {
		if ( empty( $this->get_collection_ids() ) ) return;
		
		$queue = $refresh ? $this->refresh_collection : $this->import_collection;

		$collection_ids = explode( ',', $this->get_collection_ids() );

		foreach( $collection_ids as $collection_id ) {
			$collection_id = trim( $collection_id );
			$queue->push_to_queue( $collection_id );
		}

		$queue->save()->dispatch();
	}

	/**
	 * Get the post status setting.
	 *
	 * @return bool
	 */
	public function autopublish() {
		return $this->settings['autopublish'] === 'on' ? true : false;
	}

	/**
	 * Get the desired Newswire post status.
	 *
	 * @return string
	 */
	public function get_post_status() {
		return $this->autopublish() ? 'publish' : 'draft';
	}

	/**
	 * Get the include attribution setting.
	 *
	 * @return bool
	 */
	public function include_attribution() {
		return $this->settings['include_attribution'] === 'on' ? true : false;
	}
	
	/**
	 * Get the redirect all setting.
	 *
	 * @return bool
	 */
	public function redirect_all_newswire_posts() {
		return $this->settings['redirect_all'] === 'on' ? true : false;
	}

	/**
	 * Get the collection IDs setting.
	 *
	 * @return string
	 */
	public function get_collection_ids() {
		return $this->settings['collection_ids'];
	}

	/**
	 * Get the timestamp of the last Newswire query.
	 *
	 * @param  bool $local Whether to convert the last updated time
	 *					   to Wordpress local time.
	 * @return int|bool Unix timestamp or false if never updated.
	 */
	public function get_last_update($local = false) {
		if ( $local ) {
			return $this->data['last_update'] + ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
		}

		return $this->data['last_update'];
	}

	/**
	 * Set the last updated time.
	 *
	 * @param int Unix timestamp
	 * @return void
	 */
	public function set_last_update( $timestamp ) {
		$this->data['last_update'] = (int) $timestamp;
		update_option( 'ncstate_newswire_data', $this->data );
	}

	/**
	 * Get the status of updates.
	 *
	 * @return string $message The update status message
	 */
	public function get_update_status() {
		$message = "No posts have been pulled.";

		if ( empty($this->settings['collection_ids']) ){
			$message = "No collections have been added.";
		} elseif ( $this->import_collection->is_running() ) {
			$message = "Update in progress.";
		} elseif ( $this->import_collection->is_queued() ) {
			$message = "Preparing to update.";
		} elseif ( ! $this->get_last_update() === false ) {
			$message = "Updated " . date( 'n/j/Y \a\t g:i a', Newswire::get_instance()->get_last_update( true ) );
		}

		return $message; 
	}

	/**
	 * Get the timestamp of the next Newswire query.
	 *
	 * @param  bool $local Whether to convert the next update time
	 *					   to Wordpress local time.
	 * @return int Unix timestamp.
	 */
	public function get_next_update( $local = false ) {
		$timestamp = wp_next_scheduled( 'newswire_hourly_update' );

		if ( $local && $timestamp !== false ) {
			$timestamp += ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS );
		}
		
		return date( 'n/j/Y \a\t g:i a', $timestamp );
	}


	/**
	 * Create a Newswire user if it doesn't exist.
	 *
	 * @return void
	 */
	private static function newswire_user_creation() {
		if( ! email_exists('newswire@ncsu.edu') ) {
			$user = array(
				'user_login' => 'newswire',
				'user_nicename' => 'ncstate-newswire',
				'display_name' => 'Staff',
				'role' => 'author',
				'user_email' => 'newswire@ncsu.edu',
			);
			$id = wp_insert_user( $user );
			update_user_meta( $id, 'ncsu-multiauth-realm', 'local' );
		}
	}
	
}

Newswire::get_instance();
