<?php

namespace NCState\Newswire;

use NCState\Newswire;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Uri;
use GuzzleHttp\Psr7\Request;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Exception\GuzzleException;

class Connection {

    /**
     * @var GuzzleHttp\Client
     */
    private $client;

    /**
     * @var array
     */
    public $data = array();

    /**
     * @var array
     */
    public $meta = array();

    /**
     * @var string
     */
    public $status_code;

    /**
     * Index of $data array
     *
     * @var int
     */
    private $current_post = 0;

    /**
     *
     * Construct a connection.
     *
     */
    public function __construct() {
        $this->client = new Client([
            'base_uri' => 'https://newswire.ncsu.edu', // no trailing slash
            'timeout' => 5.0,
        ]);
    }

    /**
     * Get all Newswire posts.
     *
     * @param  string $collection_id
     * @return NCState\Newswire\Connection
     */
    public function get_posts( $collection_id ) {
        $request = new Request('GET', '/api/collections/' . $collection_id . '/posts');
        
        $this->query_newswire( $request );

        return $this;
    }

    /**
     * Get only updated Newswire posts.
     *
     * @param  string $collection_id
     * @return NCState\Newswire\Connection
     */
    public function get_updates( $collection_id ) {
        $last_update = Newswire::get_instance()->get_last_update();

        if ( ! $last_update ) {
            $last_update = date('U');
        }

        $uri = new Uri('/api/collections/' . $collection_id . '/posts');

        $request = new Request(
                'GET',
                Uri::withQueryValue($uri, 'modified_since', $last_update)
            );

        $this->query_newswire( $request );

        return $this;
    }

    /**
     * Conditional for the post loop.
     *
     * @return bool
     */
    public function has_posts() {
        return ( array_key_exists( $this->current_post, $this->data ) &&
                 is_array( $this->data[$this->current_post] ) ) ||
               $this->has_next_page();
    }

    /**
     * Return the next post in response data.
     *
     * @return array
     */
    public function the_post() {
        if ( ! array_key_exists( $this->current_post, $this->data ) ) {
            $this->next_page();
        }

        $post = $this->data[$this->current_post];
        $this->current_post++;

        return $post;
    }

    /**
     * Load the next page of the last request.
     *
     * @return void
     */
    private function next_page() {
        if ( ! $this->has_next_page() ) {
            /** 
             * This could cause an infinite loop if the_post() is used outside
             * of a has_posts() loop. Maybe fix later. For now, be responsible.
             */ 
            $this->reset_post_loop();
            return;
        }

        $request = new Request(
            'GET',
            $this->get_next_page()
        );

        $this->query_newswire( $request );
    }

    /**
     * Reset the post loop to 0.
     *
     * @return void
     */
    public function reset_post_loop() {
        $this->current_post = 0;
    }

    /**
     * Does the last request have a next page?
     *
     * @return bool
     */
    public function has_next_page() {
        return ! empty( $this->meta['pagination']['links']['next'] );
    }

    /**
     * Get the next page link of the last request.
     *
     * @return string
     */
    private function get_next_page() {
        return $this->meta['pagination']['links']['next'];
    }

    /**
     * Parse a query response for the relevant data.
     *
     * @param  GuzzleHttp\Psr7\Response $response
     * @return void
     */
    private function parse_response( Response $response ) {
        $this->status_code = $response->getStatusCode();

        $array = json_decode( $response->getBody()->getContents(), true );

        $this->data = $array['data'];
        $this->meta = $array['meta'];
    }

    /**
     * Send a request to the Newswire API.
     *
     * @return void
     */
    private function query_newswire( Request $request ) {
        try {
            $response = $this->client->send( $request );
            $this->parse_response( $response );

            $this->reset_post_loop();
        } catch ( GuzzleException $e ) {
            error_log( 'Newswire Error: ' . $e->getMessage() );
        }
    }
    
}