<?php

namespace NCState\Newswire\Jobs;

use WP_Error;
use WP_Async_Request;

class Download_Image_Request extends WP_Async_Request {

    /**
     * @var string
     */
    protected $action = 'download_image';

    /**
     * Handle
     *
     * Override this method to perform any actions required
     * during the async request.
     */
    protected function handle() {
        $result = $this->download_featured_image( 
            $_POST['download_url'],
            $_POST['post_id'],
            $_POST['filename'],
            $_POST['alt_text']
        );

        if ( is_wp_error( $result ) ) {
            error_log(
                "Newswire Error: The following image failed to download - " . $_POST['filename'] .
                ". " . $result->get_error_message()
            );
        }
    }

    /**
     * Sideload an image and set it as the featured image.
     * 
     * Upload an image, attach it to a post, and set it as the featured image (the post thumbnail).
     * Additional functionality: ability to pass $post_data to override values in wp_insert_attachment
     *
     * Source: http://isabelcastillo.com/upload-image-set-featured-functions-wordpress
     * 
     * @param string $url (required) The URL of the image to download
     * @param int $post_id (required) The post ID the media is to be associated with
     * @param string $filename (required) The filename to give the image
     * @param string $alt_text (optional) The image's alt text
     * @param array $post_data (optional) Array of key => values for wp_posts table (ex: 'post_title' => 'foobar', 'post_status' => 'draft')
     * @return int|object The ID of the attachment or a WP_Error on failure
     */
     
    protected function download_featured_image( $url = null, $post_id = null, $filename = null, $alt_text = null, $post_data = array() ) {
            if ( ! $url || ! $post_id ) return new WP_Error( 'missing', "Need a valid URL and post ID..." );
            require_once( ABSPATH . 'wp-admin/includes/file.php' );

            // Download file to temp location, returns full server path to temp file, ex; /home/user/public_html/mysite/wp-content/26192277_640.tmp
            $tmp = download_url( $url );
         
            // If error storing temporarily, unlink
            if ( is_wp_error( $tmp ) ) {
                @unlink( $file_array['tmp_name'] );    // clean up
                $file_array['tmp_name'] = '';
                return $tmp; // output wp_error
            }

            $file_type = wp_check_filetype( $filename );    // determine file type (ext and mime/type)

            // If error storing temporarily, unlink
            if ( $file_type['ext'] === 'png') {
                @unlink( $file_array['tmp_name'] );    // clean up
                $file_array['tmp_name'] = '';
                return new WP_Error( 'newswire_png_error', "The Newswire plugin does not support PNG files." );
            }
         
            // assemble file data (should be built like $_FILES since wp_handle_sideload() will be using)
            $file_array['tmp_name'] = $tmp;    // full server path to temp file
     
            $file_array['name'] = $filename;
         
            // set additional wp_posts columns
            if ( empty( $post_data['post_title'] ) ) {
                $post_data['post_title'] = basename( $filename, "." . $file_type['ext'] );    // just use the original filename (no extension)
            }
         
            // make sure gets tied to parent
            if ( empty( $post_data['post_parent'] ) ) {
                $post_data['post_parent'] = $post_id;
            }
         
            // required libraries for media_handle_sideload
            require_once(ABSPATH . 'wp-admin/includes/file.php');
            require_once(ABSPATH . 'wp-admin/includes/media.php');
            require_once(ABSPATH . 'wp-admin/includes/image.php');
            
            // do the validation and storage stuff
            $att_id = media_handle_sideload( $file_array, $post_id, null, $post_data );    // $post_data can override the items saved to wp_posts table, like post_mime_type, guid, post_parent, post_title, post_content, post_status
         
            // If error storing permanently, unlink
            if ( is_wp_error( $att_id ) ) {
                @unlink( $file_array['tmp_name'] );    // clean up
                return $att_id;     // output wp_error
            }

            // Set the image Alt-Text
            if ( !empty( $alt_text ) ) {
                update_post_meta( $att_id, '_wp_attachment_image_alt', $alt_text );
            }
         
            // set as post thumbnail
            $attachment_result = set_post_thumbnail($post_id, $att_id);

            if ($attachment_result === false) {
                return new WP_Error( 'newswire_attachment_error', "There was a problem attaching the file with an ID of " . $att_id . " to the post with ID " . $post_id);
            }
         
            return $att_id;
    }

}