<?php

namespace NCState\Newswire\Jobs;

use NCState\Newswire;
use NCState\Newswire\Post;
use WP_Background_Process;
use NCState\Newswire\Connection;
use NCState\Newswire\Jobs\Download_Image_Request;

class Import_Collection extends WP_Background_Process {

    /**
     * @var string
     */
    protected $action = 'import_collection';

    /**
     * @var NCState\Newswire\Connection
     */
    protected $nw_connection;

    /**
     * @var NCState\Newswire\Jobs\Download_Image_Request
     */
    protected $image_request;


    public function __construct() {
        if ( is_multisite() ) {
            $this->action = $this->action . '_blog_' . get_current_blog_id();
        }

        parent::__construct();

        $this->nw_connection = new Connection();
        $this->image_request = new Download_Image_Request();
    }

    /**
     * Task
     *
     * Override this method to perform any actions required on each
     * queue item. Return the modified item for further processing
     * in the next pass through. Or, return false to remove the
     * item from the queue.
     *
     * @param string $collection_id
     * @return mixed
     */
    protected function task( $collection_id ) {
        $this->import_new_posts( $collection_id );
        $this->import_post_updates( $collection_id );

        return false;
    }

    /**
     * Import new posts for a collection
     *
     * @param string $collection_id
     * @return void
     */
    protected function import_new_posts( $collection_id ) {
        $posts = $this->nw_connection->get_posts( $collection_id );

        while ( $posts->has_posts() ) {
            $post = new Post( $posts->the_post() );

            if ( $post->exists() && $post->is_in_collection( $collection_id ) ) {
                /**
                 * We've put this post in this collection before,
                 * so we know we've discoverd all new posts for
                 * this import.
                 */

                break;
            }

            $post->assign_to_collection( $collection_id );
            $post->save();

            if ( $post->is_new() && $post->has_image() ) {
                $image = array(
                    'post_id' => $post->local_post->ID,
                    'download_url' => $post->raw_data['featuredImage']['data']['download_url'],
                    'filename' => $post->raw_data['featuredImage']['data']['filename'],
                    'alt_text' => $post->raw_data['featuredImage']['data']['alt_text']
                );

                $this->image_request->data( $image )->dispatch();
            }
        }
    }

    /**
     * Import updates for a collection
     *
     * @param string $collection_id
     * @return void
     */
    protected function import_post_updates( $collection_id ) {
        $posts = $this->nw_connection->get_updates( $collection_id );

        while ( $posts->has_posts() ) {
            $post = new Post( $posts->the_post() );

            $result = $post->update();

            if ( is_wp_error( $result ) ) {
                error_log( "Newswire Error: " . $result->get_error_message() );
            }
        }
    }

    /**
     * Is there a queue waiting to process?
     *
     * @return bool
     */
    public function is_queued() {
        return ! $this->is_queue_empty();
    }

    /**
     * Is the queue currently being processed?
     *
     * @return bool
     */
    public function is_running() {
        return $this->is_process_running();
    }

    /**
     * Complete
     *
     * Override if applicable, but ensure that the below actions are
     * performed, or, call parent::complete().
     */
    protected function complete() {
        parent::complete();
        Newswire::get_instance()->set_last_update( time() );
    }

}
