<?php

namespace NCState\Newswire;

use WP_Error;
use NCState\Newswire;

class Post {

    /**
     * Raw NCState Newswire API post data.
     *
     * @var array
     */
    public $raw_data;

    /**
     * The local post object for this post.
     *
     * @var WP_Post
     */
    public $local_post;

    /**
     * The post's collection IDs
     *
     * @var array
     */
    public $collections;

    /**
     * True when no local post exists upon instantiation.
     *
     * @var bool
     */
    private $is_new = false;

    /**
     * Construct a post.
     *
     * @param array $data Newswire API post data.
     */
    public function __construct( $data ) {
        $this->raw_data = $data;
        $this->local_post = $this->get_local_post();
        $this->collections = $this->get_collection_ids();
    }

    /**
     * Get the local WordPress post object that
     * corresponds to the given Newswire post data.
     *
     * @return WP_Post|null
     */
    private function get_local_post() {
        global $wpdb;
        $post = $wpdb->get_var( $wpdb->prepare(
                "SELECT $wpdb->posts.ID FROM $wpdb->posts
                LEFT OUTER JOIN $wpdb->postmeta ON $wpdb->posts.id = $wpdb->postmeta.post_id
                WHERE ( $wpdb->posts.guid=%s )
                OR ( $wpdb->postmeta.meta_key='ncstate_wire_id' AND $wpdb->postmeta.meta_value=%s )",
                array( $this->raw_data['guid'], $this->raw_data['id'] )
            )
        );

        if ( is_null( $post ) ) {
            $this->is_new = true;
            return null;
        }

        return get_post( $post );
    }

    /**
     * Save the post to the DB.
     *
     * @return NCState\Newswire\Post
     */
    public function save() {
        if ( $this->exists() ) {
            return $this->update();
        }

        $post = $this->build();

        $id = wp_insert_post( $post );
        $this->local_post = get_post( $id );

        return $this;
    }

    /**
     * Build a post array for WordPress insertion.
     *
     * @return array
     */
    private function build() {
        $post = array(
            'post_title'   => trim( $this->raw_data['post_title'] ),
            'post_excerpt' => trim( strip_tags( $this->raw_data['post_excerpt'] ) ),
            'post_status'  => Newswire::get_instance()->get_post_status(),
            'post_date'    => trim( $this->raw_data['post_published'] ),
            'post_content' => trim( $this->raw_data['post_content'] ),
            'post_author'  => $this->get_author_id(),
            'meta_input'   => array(
                'newswire_collection_ids'  => $this->collections,
                'newswire_enforce_redirect'=> $this->raw_data['redirect'],
                'newswire_source_url'      => $this->raw_data['canonical_url'],
                'newswire_post_guid'       => $this->raw_data['guid'],
                'newswire_publisher'       => $this->raw_data['contributor']['data']['name'],
                'ncstate_wire_id'          => $this->raw_data['id'],
                'youtube_id'               => $this->raw_data['youtube_id'],
                'flickr_id'                => $this->raw_data['flickr_id'],
                'source'                   => 'ncstate_wire'
            )
        );

        return $post;
    }

    /**
     * Update an existing post in the DB.
     *
     * @return NCState\Newswire\Post
     */
    public function update() {
        if ( ! $this->exists() ) {
            $message = "Newswire post " . $this->raw_data['id'] . " could not be updated. The post does not exist locally.";
            return new WP_Error( 'newswire_post_error', $message );
        };

        $post = $this->local_post;

        $post->meta_input = array(
            'newswire_collection_ids' => $this->collections
        );

        if ( ! $this->is_source() ) {
            $post->post_content = trim( $this->raw_data['post_content'] );
            $post->post_author  = $this->get_author_id();
        }

        $id = wp_update_post( $post );
        $this->local_post = get_post( $id );

        return $this;
    }

    /**
     * Add source attribution filter. Filter is hooked in plugin init.
     *
     * @param  string $content From WordPress
     * @return string
     */
    public static function maybe_add_attribution( $content ) {
        $postmeta = get_post_meta( $GLOBALS['post']->ID );
        $is_from_newswire = isset($postmeta['source'][0])
                            && $postmeta['source'][0] === 'ncstate_wire';

        if ( $is_from_newswire && Newswire::get_instance()->include_attribution() ) {
            $content .= '<p><em>This post was <a href="' . $postmeta['newswire_source_url'][0] . '">originally published</a> in ' . $postmeta['newswire_publisher'][0] . '.</em></p>';
        }

        return $content;
    }

    /**
     * Is this site the original source of the post?
     *
     * @return bool
     */
    private function is_source() {
        return $this->raw_data['guid'] === $this->local_post->guid;
    }

    /**
     * Getter for is_new property.
     *
     * @return bool
     */
    public function is_new() {
        return $this->is_new;
    }

    /**
     * Does this post exist in the database?
     *
     * @return bool
     */
    public function exists() {
        return ! is_null( $this->local_post );
    }

    /**
     * Get the post's stored collection IDs.
     *
     * @return array
     */
    private function get_collection_ids() {
        $ids = array();

        if ( $this->exists() ) {
            $ids = get_post_meta(
                $this->local_post->ID,
                'newswire_collection_ids',
                true
            );
        }

        return is_array( $ids ) ? $ids : array();
    }

    /**
     * Assign this post to a collection.
     *
     * @param  string $collection_id
     * @return void
     */
    public function assign_to_collection( $collection_id ) {
        if ( ! $this->is_in_collection( $collection_id ) ) {
            $this->collections[] = $collection_id;
        }
    }

    /**
     * Check if this post is in a certain collection.
     *
     * @param  string $collection_id
     * @return bool
     */
    public function is_in_collection( $collection_id ) {
        return in_array( $collection_id, $this->collections );
    }

    /**
     * Check if this post has a featured image.
     *
     * @return bool
     */
    public function has_image() {
        return array_key_exists( 'featuredImage', $this->raw_data );
    }

    /**
     * Get the local author ID by email or default to
     * the Newswire user.
     *
     * @return string
     */
    private function get_author_id() {
        $email = $this->raw_data['author']['data']['email'];
        return email_exists( $email ) ? email_exists( $email ) : email_exists( 'newswire@ncsu.edu' );
    }

}
