<?php
/*
Based on the the following plugin:

Plugin Name: Page Links To
Plugin URI: http://txfx.net/wordpress-plugins/page-links-to/
Author: Mark Jaquith
Author URI: http://coveredwebservices.com/
*/

namespace NCState\Newswire;

use \NCState\Newswire;
use WP_Stack_Plugin;

class Redirect_Manager extends WP_Stack_Plugin {
    static $instance;
    const LINK_META_KEY = 'newswire_source_url';
    const REDIRECTS_META_KEY = '_newswire_redirects';
    const ENFORCED_REDIRECT_META_KEY = 'newswire_enforce_redirect';

    public function __construct() {
        self::$instance = $this;
        $this->hook( 'init' );
    }

    /**
     * Registers all the hooks.
     */
    function init() {
        // Hook in to URL generation
        $this->hook( 'post_link',       'link', 20 );

        // Non-standard priority hook
        $this->hook( 'do_meta_boxes', 20 );

        // Non-standard callback hook
        $this->hook( 'load-post.php', 'load_post' );

        // Standard hooks
        $this->hook( 'template_redirect' );
        $this->hook( 'save_post' );

        // Metadata validation grants users editing privileges for our custom fields
        register_meta('post', self::REDIRECTS_META_KEY, null, '__return_true');
    }

    /**
     * Returns a single piece of post meta
     * @param  integer $post_id a post ID
     * @param  string $key a post meta key
     * @return string|false the post meta, or false, if it doesn't exist
     */
    function get_post_meta( $post_id, $key ) {
        $meta = get_post_meta( absint( $post_id ), $key, true );
        if ( '' === $meta )
            return false;
        return $meta;
    }

    /**
     * Returns the link for the specified post ID if set to redirect
     *
     * @param  integer $post_id a post ID
     * @return mixed either a URL or false
     */
    function get_link( $post_id ) {
        if ( Newswire::get_instance()->redirect_all_newswire_posts()
            || $this->get_post_meta( $post_id, self::REDIRECTS_META_KEY )
            || $this->get_post_meta( $post_id, self::ENFORCED_REDIRECT_META_KEY ) ) {
            return $this->get_post_meta( $post_id, self::LINK_META_KEY );
        }

        return false;
    }

    /**
     * Whether the given post is from Newswire
     *
     * @param  integer $post_id a post ID
     * @return bool
     */
    function is_newswire_post( $post_id ) {
        return 'ncstate_wire' === $this->get_post_meta( $post_id, 'source', true );
    }

    /**
     * Adds the meta box to the post edit screen
     *
     * @param string $page the name of the current page
     * @param string $context the current context
     * @param mixed $object post object, if available
     */
    function do_meta_boxes( $page, $context, $object ) {
        if ( is_object( $object ) && $this->is_newswire_post( $object->ID ) && 'side' === $context )
            add_meta_box( 'newswire-post-settings', 'Newswire Post Settings', array( $this, 'meta_box' ), $page, 'side', 'low' );
    }

    /**
     * Outputs the Newswire Post Settings meta box if post is from Newswire
     */
    function meta_box() {
        $null = null;
        $post = get_post( $null );

        if ( Newswire::get_instance()->redirect_all_newswire_posts() ) {
    ?>
        <p>You have enabled redirection for all Newswire posts. If you would
        like to control redirection on individual posts, please visit the
        <a href="options-general.php?page=ncstate_newswire_settings">Newswire Settings</a>
        page and disable the "Redirect all Newswire posts" option.</p>
<?php
        } elseif ($this->get_post_meta($post->ID, self::ENFORCED_REDIRECT_META_KEY)) {
?>
        <p>Due to theme and formatting inconsistencies this post has been redirected to its original source.</p>
    <?php
        } else {
            echo '<p>';
            wp_nonce_field( 'newswire_' . $post->ID, '_newswire_nonce', false, true );
            echo '</p>';
            $url = $this->get_link( $post->ID );
            if ( ! $url ) {
                $redirected = false;
            } else {
                $redirected = true;
            }
    ?>
        <p><label><input type="checkbox" id="newswire-redirect-choose-wp" name="newswire_redirect_choice" value="1" <?php checked( $redirected ); ?> />Redirect to source URL</label></p>
    <?php
        }
    }

    /**
     * Saves data on post save
     *
     * @param int $post_id a post ID
     * @return int the post ID that was passed in
     */
    function save_post( $post_id ) {
        if ( isset( $_REQUEST['_newswire_nonce'] ) && wp_verify_nonce( $_REQUEST['_newswire_nonce'], 'newswire_' . $post_id ) ) {
            if ( ( isset( $_POST['newswire_redirect_choice'] ) && '1' === $_POST['newswire_redirect_choice'] ) ) {
                $this->set_to_source( $post_id );
            } else {
                $this->set_to_local( $post_id );
            }
        }
        return $post_id;
    }

    /**
     * Have a post redirect to the source URL
     *
     * @param int $post_id post ID
     * @return bool whether anything changed
     */
    function set_to_source( $post_id ) {
        return (bool) update_post_meta( $post_id, self::REDIRECTS_META_KEY, '1' );
    }

    /**
     * Discard the redirect setting and point a post to its normal URL
     *
     * @param int $post_id post ID
     * @return bool whether the link was deleted
     */
    function set_to_local( $post_id ) {
        return delete_post_meta( $post_id, self::REDIRECTS_META_KEY );
    }

    /**
     * Filter for Post links
     *
     * @param string $link the URL for the post or page
     * @param int|WP_Post $post post ID or object
     * @return string output URL
     */
    function link( $link, $post ) {
        $post = get_post( $post );

        $meta_link = $this->get_link( $post->ID );

        if ( $meta_link ) {
            $link = esc_url( $meta_link );
        }

        return $link;
    }

    /**
     * Performs a redirect
     */
    function template_redirect() {
        $link = $this->get_redirect();

        if ( ! $link )
            return;

        wp_redirect( $link, 301 );
        exit;
    }

    /**
     * gets the redirection URL
     *
     * @return string|bool the redirection URL, or false
     */
    function get_redirect() {
        if ( ! is_singular() )
            return false;

        if ( ! get_queried_object_id() )
            return false;

        $link = $this->get_link( get_queried_object_id() );

        return $link;
    }

    /**
     * Hooks in as a post is being loaded for editing and conditionally adds a notice
     */
    function load_post() {
        if ( isset( $_GET['post'] ) ) {
            if ( $this->get_link( (int) $_GET['post'] ) )
                $this->hook( 'admin_notices', 'notify_of_source_link' );
        }
    }

    /**
     * Outputs a notice that the current post item is pointed to the source URL
     */
    function notify_of_source_link() {
        ?><div class="updated"><p><strong>Note</strong>: This post is redirecting to the orignal source. Use the &#8220;Newswire Post Settings&#8221; box to change this behavior.</p></div><?php
    }

}
